/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.router;

import java.util.UUID;

import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;
import org.mapstruct.ReportingPolicy;

import com.google.protobuf.ByteString;

import de.ozgcloud.eingang.common.formdata.Antragsteller;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;
import de.ozgcloud.vorgang.common.grpc.GrpcFormDataMapper;
import de.ozgcloud.vorgang.vorgang.GrpcAntragsteller;
import de.ozgcloud.vorgang.vorgang.GrpcEingang;
import de.ozgcloud.vorgang.vorgang.GrpcIncomingFileGroup;
import de.ozgcloud.vorgang.vorgang.GrpcZustaendigeStelle;

@Mapper(unmappedTargetPolicy = ReportingPolicy.WARN, //
		unmappedSourcePolicy = ReportingPolicy.WARN, //
		nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, //
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS, //
		collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED, //
		uses = { GrpcFormDataMapper.class, ServiceKontoMapper.class })
public interface GrpcEingangMapper {

	@Mapping(source = "antragsteller.data", target = "antragsteller.otherData")
	@Mapping(source = "attachments", target = "attachmentsList")
	@Mapping(source = "representations", target = "representationsList")
	GrpcEingang toEingang(FormData formData);

	default ByteString byteArrayToByteString(byte[] byteArray) {
		if (byteArray == null) {
			return null;
		}

		return ByteString.copyFrom(byteArray);
	}

	@Mapping(source = "files", target = "filesList")
	GrpcIncomingFileGroup map(IncomingFileGroup fileGroup);

	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "bezeichnungBytes", ignore = true)
	@Mapping(target = "emailBytes", ignore = true)
	@Mapping(target = "organisationseinheitenIdBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	GrpcZustaendigeStelle toZustaendigeStelle(ZustaendigeStelle zustaendigeStelle);

	GrpcAntragsteller toAntragsteller(Antragsteller antragsteller);

	default String uuidToString(UUID id) {
		return id.toString();
	}

	// FIXME map representations and attachments
	@Mapping(target = "attachment", ignore = true)
	@Mapping(target = "attachments", ignore = true)
	@Mapping(target = "representation", ignore = true)
	@Mapping(target = "representations", ignore = true)
	// TOASK: Wird aktuell nicht gebraucht, trotzdem implementiern?
	@Mapping(target = "header.serviceKonto", ignore = true)
	FormData toFormData(GrpcEingang eingang);

}
